#include <iostream>

#include "config.h" // For HAVE_LIBPTHREADS
#include "argyll_support.h"
#include "channelresponse.h"
#include "1dinterp.h"

using namespace std;

RawChannelResponses::RawChannelResponses(const char *filename)
	: TI3File(filename), DeviceNColorantList(), firstchannel(NULL), paper()
{
	// Deduce paper-white from the data.  This only works for Subtractive colour,
	// since we check for colorants all set to zero.  Since there are commonly several
	// unprinted patches, we average them.

	int papercount=0;
	for(int i=0;i<GetSets();++i)
	{
		double acc=0.0;
		double **data=GetData(i);
		for(int j=0;j<colorants;++j)
		{
			acc+=*data[j];
		}
		if(acc==0.0)
		{
			paper.X+=*data[Xindex];
			paper.Y+=*data[Yindex];
			paper.Z+=*data[Zindex];
			++papercount;
		}
	}
	if(papercount)
	{
		paper.X/=papercount;
		paper.Y/=papercount;
		paper.Z/=papercount;
	}
	cerr << papercount << " unprinted patches found - Paper colour (XYZ): " << paper.X << ", " << paper.Y << ", " << paper.Z << endl;

	// Generate the channel responses - one for each colorant
	for(int i=0;i<colorants;++i)
	{
		new ChannelResponse(this,i);
	}
}


RawChannelResponses::~RawChannelResponses()
{
	while(firstchannel)
		delete firstchannel;
}


ChannelResponse *RawChannelResponses::FirstChannel()
{
	return(firstchannel);
}


ChannelResponseSample *RawChannelResponses::GetPaper()
{
	return(&paper);
}

////////////////////////////////////////////////////////////////////////////////////////////////


ChannelResponse::ChannelResponse(RawChannelResponses *header,int channel)
	: Interpolate_1D(), channelindex(channel), channelname(NULL), samplecount(0), samples(NULL),
	header(header), nextchannel(NULL), prevchannel(NULL), interpin(NULL), interpout(NULL)
{
	// We sort out the colorant stuff first because if an unknown colorant is going
	// to trigger an exception in the DeviceNColorant constructor we want it to
	// happen before we're linked into the list.
	channelname=header->GetFieldName(channel);
	cerr << "Created channel for: " << channelname << endl;
	const char *sn=GetShortName();
	const char *longname=ArgyllSupport::GetColorantLongName(sn);
	cerr << "Creating colorant for " << longname << endl;
	new DeviceNColorant(*header,longname);


	if((prevchannel=header->firstchannel))
	{
		while(prevchannel->nextchannel)
			prevchannel=prevchannel->nextchannel;
		prevchannel->nextchannel=this;
	}
	else
		header->firstchannel=this;


	// We count the number of patches which involve just this channel

	for(int i=0;i<header->GetSets();++i)
	{
		double chan=0.0;
		double acc=0.0;
		double **data=header->GetData(i);
		for(int j=0;j<header->colorants;++j)
		{
			if(j==channelindex)
				chan=*data[j];
			else
				acc+=*data[j];
		}
		// Chan contains the contents of the row for this channel
		// acc contains the sum total of the other colorants' values.
		if(acc<0.01 && chan>0.01)
		{
			++samplecount;
		}
	}
	++samplecount;	// The count calculated above excludes the paper colour

	cerr << "Found " << samplecount << "patches in total" << endl;
	samples=new ChannelResponseSample[samplecount];
	interpin=new double[samplecount];
	interpout=new double[samplecount];
	int idx=0;
	samples[idx]=header->paper;
	++idx;

	LabValue l(samples[0]);
	float lmin=l.L;
	float lmax=l.L;
	float bmin=l.b;
	float bmax=l.b;

	for(int i=0;i<header->GetSets();++i)
	{
		double chan=0.0;
		double acc=0.0;
		double **data=header->GetData(i);
		for(int j=0;j<header->colorants;++j)
		{
			if(j==channelindex)
				chan=*data[j];
			else
				acc+=*data[j];
		}
		// Chan contains the contents of the row for this channel
		// acc contains the sum total of the other colorants' values.
		if(acc<0.01 && chan>0.01)
		{
			samples[idx].input=chan;
			samples[idx].X=*data[header->Xindex];
			samples[idx].Y=*data[header->Yindex];
			samples[idx].Z=*data[header->Zindex];

			LabValue l(samples[idx]);
			if(l.L < lmin) lmin=l.L;
			if(l.L > lmax) lmax=l.L;
			if(l.b < bmin) bmin=l.b;
			if(l.b > bmax) bmax=l.b;

			++idx;
		}
	}
	cerr << "L* range: " << lmin << " - " << lmax << endl;
	cerr << "b range: " << bmin << " - " << bmax << endl;
	if((bmax-bmin)/3.0 > (lmax-lmin))
	{
		cerr << "b has greater range than L*" << endl;
		for(idx=0;idx<samplecount;++idx)
		{
			interpin[idx]=samples[idx].input;
			LabValue l(samples[idx]);
			interpout[idx]=100.0*(l.b-bmin)/(bmax-bmin);
		}
	}
	else
	{
		for(idx=0;idx<samplecount;++idx)
		{
			interpin[idx]=samples[idx].input;
			LabValue l(samples[idx]);
			interpout[idx]=100.0*(lmax-l.L)/(lmax-lmin);
		}
	}
	Populate(samplecount,interpin,interpout,.001);	// minimal smoothing
}


ChannelResponse::~ChannelResponse()
{
	if(interpin)
		delete[] interpin;
	if(interpout)
		delete[] interpout;
	if(samples)
		delete[] samples;
	cerr << "Removing channel: " << GetShortName() << endl;
	if(prevchannel)
		prevchannel->nextchannel=nextchannel;
	else
		header->firstchannel=nextchannel;
	if(nextchannel)
		nextchannel->prevchannel=prevchannel;
}


ChannelResponse *ChannelResponse::NextChannel()
{
	return(nextchannel);
}


ChannelResponse *ChannelResponse::PrevChannel()
{
	return(prevchannel);
}


const char *ChannelResponse::GetName()
{
	return(channelname);
}


const char *ChannelResponse::GetLongName()
{
	return(ArgyllSupport::GetColorantLongName(GetShortName()));
}


const char *ChannelResponse::GetShortName()
{
	if(channelname)
	{
		int i=0;
		while(channelname[i])
		{
			if(channelname[i]=='_')
			{
				return(channelname+i+1);
			}
			++i;
		}
	}
	return(NULL);
}


int ChannelResponse::GetChannelIndex()
{
	return(channelindex);
}

int ChannelResponse::GetSampleCount()
{
	return(samplecount);
}

ChannelResponseSample *ChannelResponse::GetSamples()
{
	return(samples);
}

