#include <iostream>
#include <string.h>

#include <gutenprint/gutenprint.h>

#include "support/util.h"
#include "gpdevicensupport.h"

#define USE_GUTENPRINT_BRANCH

using namespace std;


int GetPrinterColorantCount(stp_vars_t *vars)
{
	int result=0;
	stp_parameter_t desc;
	stp_describe_parameter(vars,"ChannelNames",&desc);
	stp_string_list_t *strlist=desc.bounds.str;
	if(strlist)
		result=stp_string_list_count(strlist);
	stp_parameter_description_destroy(&desc);
	return(result);
}


//////////////////////////////////////////////////////////////////////////////


GPColorantList::GPColorantList(stp_vars_t *vars) : DeviceNColorantList()
{
	BuildList(vars);
}


GPColorantList::GPColorantList(GPrinterSettings *gp) : DeviceNColorantList()
{
	BuildList(gp->stpvars);
}


GPColorantList::~GPColorantList()
{
}


GPColorant *GPColorantList::First()
{
	return((GPColorant *)FirstColorant());
}


// Create a string representation of which colorants are active,
// suitable for storing in a config file.

char *GPColorantList::GetEnabledColorants()
{
	char *result=NULL;
	GPColorant *col=First();
	int len=2;  // Initial ":" and null-terminator
	while(col)
	{
		if(col->GetEnabled())
		{
			const char *name=col->GetName();
			if(name)
				len+=strlen(name)+2;
		}
		col=col->NextColorant();
	}
	result=(char *)malloc(len);
	result[0]=':';
	result[1]=0;
	col=First();
	while(col)
	{
		if(col->GetEnabled())
		{
			const char *name=col->GetName();
			if(name)
			{
				strcat(result,name);
				strcat(result,":");
			}
		}
		col=col->NextColorant();
	}
	if(!result)
		result=strdup("");
	return(result);
}


// Given a string respresention as produced by GetEnabledColorants()
// sets the "enabled" flag in the colorants

void GPColorantList::SetEnabledColorants(const char *colstr)
{
	if(colstr && strlen(colstr))
	{
		GPColorant *col=First();
		while(col)
		{
			col->Disable();
			const char *name=col->GetName();
			if(name && strlen(name))
			{
				const char *tmp=strstr(colstr,name);
				while(tmp)
				{
					if(tmp[-1]==':' && tmp[strlen(name)]==':')
					{
						col->Enable();
						tmp=NULL;
					}
					else
						tmp=strstr(tmp+1,name);
				}
			}
			col=col->NextColorant();
		}
	}
}


// Builds a DeviceNColorantList from an stp_vars_t's
// ChannelNames variable.
// Dummy entries will be created for unknown channels,
// to retain the 1:1 mapping between colorant index and
// printer raw channels.
// Should no longer be necessary since 2009-01-04

static const char *ChannelOrder[]=
{
	"Black","Matte Black","Photo Black","Light Black","Light Light Black","Cyan","Light Cyan",
	"Magenta","Light Magenta","Yellow","Red","Blue","Gloss Optimizer",NULL
};


void GPColorantList::BuildList(stp_vars_t *vars)
{
	const char *mode=stp_get_string_parameter(vars,"InputImageType");
	if(!mode)
	{
		stp_set_string_parameter(vars,"InputImageType","CMYK");
		mode="CMYK";
	}
	if(strcmp(mode,"Raw")==0)
	{
		BuildList_Raw(vars);
	}
	else if(strcmp(mode,"RGB")==0)
	{
		new GPColorant(*this,"Red");
		new GPColorant(*this,"Green");
		new GPColorant(*this,"Blue");
	}
	else if(strcmp(mode,"CMYK")==0)
	{
		new GPColorant(*this,"Cyan");
		new GPColorant(*this,"Magenta");
		new GPColorant(*this,"Yellow");
		new GPColorant(*this,"Black");
	}
}


void GPColorantList::BuildList_Raw(stp_vars_t *vars)
{
	// Set printer to use all available colorants before asking for channel names...
	char chans[4]={0};
	sprintf(chans,"%d",GetPrinterColorantCount(vars));
	cerr << "Setting rawchannels to:" << chans << endl;
	stp_set_string_parameter(vars,"RawChannels",chans);
	stp_parameter_t desc;

	// We try to use the new RawChannelNames parameter from Gutenprint_5_2_branch
	stp_describe_parameter(vars,"RawChannelNames",&desc);
	stp_string_list_t *strlist=desc.bounds.str;
	if(strlist)
	{
		int ncol=stp_string_list_count(strlist);
		cerr << "Have " << ncol << " raw channels" << endl;
		cerr << "(Printer has " << GetPrinterColorantCount(vars) << " ink channels in total)" << endl;
		for(int j=0;j<ncol;++j)
		{
			stp_param_string_t *p=stp_string_list_param(strlist,j);
			cerr << "Have " << p->name << " : " << p->text << endl;
			try
			{
				new GPColorant(*this,p->name,p->text);
			}
			catch(const char *err)
			{
				cerr << "Ignoring colorant: " << p->name << endl;
				new GPColorant(*this);	// We include a dummy colorant for unrecognised
										// channels (gloss optimiser, mainly) so that
										// each colorant's position in the list continues
										// to match its raw channel number.
			}
		}
	}
	else
	{
		cerr << "Falling back to ChannelNames[]" << endl;
		stp_describe_parameter(vars,"ChannelNames",&desc);
		stp_string_list_t *strlist=desc.bounds.str;
		if(strlist)
		{
			int i=0;
			int ncol=stp_string_list_count(strlist);
			cerr << "Have " << ncol << " raw channels" << endl;
			while(ChannelOrder[i])
			{
				for(int j=0;j<ncol;++j)
				{
					stp_param_string_t *p=stp_string_list_param(strlist,j);
					cerr << "Comparing " << p->name << " against " << ChannelOrder[i] << endl;
					if(StrcasecmpIgnoreSpaces(p->name,ChannelOrder[i])==0)
					{
						try
						{
							cerr << "-> Adding " << p->name << endl;
							new GPColorant(*this,p->name);
						}
						catch(const char *err)
						{
							cerr << "Ignoring colorant: " << p->name << endl;
							new GPColorant(*this);	// We include a dummy colorant for unrecognised
													// channels (gloss optimiser, mainly) so that
													// each colorant's position in the list continues
													// to match its raw channel number.
						}
						j=ncol;
					}
				}
				++i;
			}
		}
	}
	stp_parameter_description_destroy(&desc);
}


#if 0
void GPColorantList::BuildList(stp_vars_t *vars)
{
	stp_parameter_t desc;
	stp_describe_parameter(vars,"ChannelNames",&desc);
	stp_string_list_t *strlist=desc.bounds.str;
	if(strlist)
	{
		int ncol=stp_string_list_count(strlist);
		cerr << "Have " << ncol << " raw channels" << endl;
		for(int j=0;j<ncol;++j)
		{
			stp_param_string_t *p=stp_string_list_param(strlist,j);
			cerr << "Ink: " << p->name << endl;
			try	
			{
				new GPColorant(*this,p->name);
			}
			catch(const char *err)
			{
				cerr << "Ignoring colorant: " << p->name << endl;
				new GPColorant(*this);	// We include a dummy colorant for unrecognised
										// channels (gloss optimiser, mainly) so that
										// each colorant's position in the list continues
										// to match its raw channel number.
			}
		}
	}
	stp_parameter_description_destroy(&desc);
}
#endif


/////////////////////////////////////////////////////////////////////////////


GPColorant::GPColorant(DeviceNColorantList &header,const char *name, const char *displayname)
	: DeviceNColorant(header,name), enabled(true), displayname(NULL)
{
	if(displayname)
		this->displayname=strdup(displayname);
	else
		this->displayname=strdup(gettext(name));
}


// Dummy entry
GPColorant::GPColorant(DeviceNColorantList &header) : DeviceNColorant(header,NULL,255,255,255), enabled(false), displayname(NULL)
{
}


GPColorant::~GPColorant()
{
	if(displayname)
		free(displayname);
}


void GPColorant::Enable()
{
	enabled=true;
}


void GPColorant::Disable()
{
	enabled=false;
}


bool GPColorant::GetEnabled()
{
	return(enabled);
}

GPColorant *GPColorant::NextColorant()
{
	return((GPColorant *)DeviceNColorant::NextColorant());
}

GPColorant *GPColorant::PrevColorant()
{
	return((GPColorant *)DeviceNColorant::PrevColorant());
}

const char *GPColorant::GetDisplayName()
{
	return(displayname);
}

