#include <iostream>

#include <getopt.h>

#include <gtk/gtk.h>

#include "support/progresstext.h"
#include "support/progressbar.h"

#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_devicen_remap.h"
#include "imagesource/imagesource_scaledensity.h"

#include "testpatterns.h"

#include "lincore.h"
#include "linproject.h"
#include "linearizedprinter.h"
#include "channelresponse.h"

#include "config.h"


using namespace std;


class Options
{
	public:
	Options(int argc, char *argv[]) : presetname(NULL), ti1name(NULL), ti3name(NULL)
	{
		static struct option long_options[] =
		{
			{"help",no_argument,NULL,'h'},
			{"version",no_argument,NULL,'v'},
			{"preset",required_argument,NULL,'p'},
			{"ti1",required_argument,NULL,'t'},
			{"ti3",required_argument,NULL,'T'},
			{"density",required_argument,NULL,'d'},
			{"gamma",required_argument,NULL,'g'},
			{0, 0, 0, 0}
		};

		while(1)
		{
			int c;
			c = getopt_long(argc,argv,"hvp:t:T:d:g:",long_options,NULL);
			if(c==-1)
				break;
			switch (c)
			{
				case 'h':
					Usage(argc,argv);
					throw 0;
					break;
				case 'v':
					cerr << "PrintDeviceN - part of " << PACKAGE_STRING << endl;
					throw 0;
					break;
				case 'p':
					presetname=optarg;
					break;
				case 't':
					ti1name=optarg;
					break;
				case 'T':
					ti3name=optarg;
					break;
			}
		}
	}
	~Options()
	{
	}
	void Usage(int argc,char *argv[])
	{
		cerr << "Usage: " << argv[0] << " [options] Image1.tif [Image2.tif] ..." << endl;
		cerr << "\t -h --help\t\tdisplay this message" << endl;
		cerr << "\t -v --version\t\tdisplay version" << endl;
		cerr << "\t -p --preset\t\tfetch printer settings from a" << endl;
		cerr << "\t\t\t\tPhotoPrint preset or .gplin file" << endl;
		cerr << "\t -t --ti1\t\tuse an Argyll .ti1 file for colorant remapping" << endl;
		cerr << "\t -T --ti3\t\tuse an Argyll .ti3 file for channel linearization" << endl;
	}
	const char *presetname;
	const char *ti1name;
	const char *ti3name;
};


int main(int argc, char **argv)
{
	stp_init();
	gtk_init(&argc,&argv);
	try
	{
		Options opts(argc,argv);

		if(!opts.presetname)
			throw "Please provide a printer preset file with the -p argument";

		LinearizationCore core;
		LinProject project(core,opts.presetname);
		project.Load();

		GPColorantList colorants(&project.gprinter);
		int colcount=colorants.GetColorantCount();

		int *channelmap=NULL;

		char *fn=NULL;
		if(opts.ti3name)
			fn=strdup(opts.ti3name);
		else
			fn=project.GetTempRoot("RawLin",".ti3");
		try
		{
			project.gprinter.SetCurvesFromTI3(fn);
			cerr << "Using linearization curves derived from " << fn << endl;
		}
		catch(const char *err)
		{	
			cerr << "Error: " << err << endl;
			cerr << "TI3 file for linearization curves not available" << endl;
		}
		if(fn)
			free(fn);

		if(opts.ti1name)
		{
			TI1File ti1(opts.ti1name);
			channelmap=new int[colcount];
			for(int i=0;i<colcount;++i)
				channelmap[i]=0;

			DeviceNColorant *col=colorants.FirstColorant();
			int i=0;
			while(col)
			{
				try
				{
					int idx=ti1.GetColorantIndex(col->GetName());
					if(idx>-1)
						channelmap[idx]=i;
					else
						cerr << "-1: Channel not found" << endl;
				}
				catch(const char *err)
				{
					cerr << "Warning: " << err << endl;
				}
				col=col->NextColorant();
				++i;
			}
			for(i=0;i<colcount;++i)
				cerr << "Mapping " << i << " to " << channelmap[i] << endl;
		}

		if(argc>optind)
		{
			for(int file=optind;file<argc;++file)
			{
				try
				{
					cerr << "Printing " << argv[file] << endl;

					ImageSource *is=ISLoadImage(argv[file]);

					if(channelmap)
					{
						is=new ImageSource_ToDeviceN(is);
						is=new ImageSource_DeviceN_Remap(is,channelmap,colcount);
						cerr << "Have a channel map" << endl;
					}
					else
						cerr << "No channel map" << endl;

					ProgressBar p("Printing...",true);
					project.gprinter.SetProgress(&p);
//					project.gprinter.Print(&dst,0,0);
					project.gprinter.Print(is,0,0);
					delete is;
				}
				catch(const char *err)
				{
					cerr << "Printing failed: " << err << endl;
				}
			}
		}
		if(channelmap)
			delete[] channelmap;

		cerr << "PrintDeviceN" << endl;
	}
	catch(const char *err)
	{
		cerr << "Error: " << err << endl;
	}
	catch(int)
	{
	}
	return(0);
}

