#include <iostream>

#include "miscwidgets/generaldialogs.h"
#include "support/progressbar.h"
#include "imageutils/jpegsave.h"
#include "support/util.h"
#include "stpui_widgets/stpui_combo.h"

#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_scaledensity.h"
#include "imagesource/imagesource_modifiedgamma.h"
#include "imagesource/imagesource_gamma.h"
#include "imagesource/imagesource_devicen_remap.h"

#include "testpatterns.h"
#include "colorantselector.h"

#include "stepwidget_basegamma.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)


// Base gamma "step" - self contained class which builds and links all
// the widgets required.  Use the "widget" member to obtain a GtkWidget which can
// be packed into a window


StepWidget_BaseGamma::StepWidget_BaseGamma(ProjectGUI *gui) : StepWidget(gui), colorants(&gui->gprinter)
{
	widget=gtk_vbox_new(FALSE,0);

	GtkWidget *hbox=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(widget),hbox,TRUE,TRUE,0);
	gtk_widget_show(hbox);


	// Colorantselector
	cerr << "Finding Colorants" << endl;
	colorants.SetEnabledColorants(gui->FindString("Colorants"));
	cerr << "Done" << endl;
	colorantsel=colorantselector_new(&colorants);
	gtk_box_pack_start(GTK_BOX(hbox),colorantsel,FALSE,FALSE,0);
	gtk_widget_set_sensitive(colorantsel,false);
	gtk_widget_show(colorantsel);
	

	GtkWidget *vbox=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox),vbox,TRUE,TRUE,8);
	gtk_widget_show(vbox);


	GtkWidget *table=gtk_table_new(3,2,FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table),8);

	gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,0);
	gtk_widget_show(table);


	// Spacer box
	GtkWidget *tmp=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox),tmp,TRUE,TRUE,0);
	gtk_widget_show(tmp);


	// Papersize selector
	
	tmp=gtk_label_new(_("Paper size:"));
	gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,1,0,1);
	gtk_widget_show(tmp);

	papersize=stpui_combo_new(gui->gprinter.stpvars,"PageSize",NULL);
	gtk_table_attach_defaults(GTK_TABLE(table),papersize,1,2,0,1);
	gtk_widget_show(papersize);


	// Starting Gamma

	tmp=gtk_label_new(_("Initial gamma:"));
	gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,1,1,2);
	gtk_widget_show(tmp);

	initialgamma=gtk_spin_button_new_with_range(0,1.0,0.005);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(initialgamma),gui->gprinter.linearization.FindFloat("BaseGamma"));
	g_signal_connect(G_OBJECT(initialgamma),"changed",G_CALLBACK(markmodified),this);
	gtk_table_attach_defaults(GTK_TABLE(table),initialgamma,1,2,1,2);
	gtk_widget_show(initialgamma);


	// Print button

	tmp=gtk_button_new_with_label(_("Print Gamma Evaluation Chart"));
	gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,2,2,3);
	gtk_widget_show(tmp);

	g_signal_connect(G_OBJECT(tmp),"clicked",G_CALLBACK(printclicked),this);

	// Corrected Gamma

	tmp=gtk_label_new(_("Correction value\n(read from print):"));
	gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,1,3,4);
	gtk_widget_show(tmp);

	correctedgamma=gtk_spin_button_new_with_range(0,1.0,0.05);
	gtk_table_attach_defaults(GTK_TABLE(table),correctedgamma,1,2,3,4);
	float corrg=0.5;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(correctedgamma),corrg);
	g_signal_connect(G_OBJECT(correctedgamma),"changed",G_CALLBACK(markmodified),this);
	gtk_widget_show(correctedgamma);
}


// Text for the Directions box

const char *StepWidget_BaseGamma::GetDirections()
{
	return(_("Please use this page to determine a suitable "
		"base gamma.  Select an initial gamma (probably around 0.5 or so) "
		"then click the Print button.  Read off on the printed chart the position "
		"at which the gradient appears to match the checkerboard in density.  You "
		"might find the 'Squint Test' useful for this!\n\n"
		"Accuracy is not critical here - this is merely used to ensure the test "
		"patches are well-distributed in the next step."));
}


void StepWidget_BaseGamma::SaveChanges()
{
	const char *ps=stp_get_string_parameter(gui->gprinter.stpvars,"PageSize");
	cerr << "Pagesize is currently set to: " << ps << endl;
	float initg=gtk_spin_button_get_value(GTK_SPIN_BUTTON(initialgamma));
	float corrg=gtk_spin_button_get_value(GTK_SPIN_BUTTON(correctedgamma));

//	float corrx=1.0-pow(1.0-corrg,initg); // Correct for original gamma value
//	float gamma=log(1.0-corrx)/log(0.5);
	float corrx=1.0-ImageSource_ModifiedGamma::ModifiedGamma(1.0-corrg,initg);
	float gamma=ImageSource_ModifiedGamma::FindGamma(0.5,1.0-corrx);
	cerr << "Setting gamma to:" << gamma << endl;
	gui->gprinter.linearization.SetFloat("BaseGamma",gamma);

	char *cols=colorants.GetEnabledColorants();
	cerr << "Selected colorants: " << cols << endl;
	gui->SetString("Colorants",cols);
	free(cols);

	gui->Save();

	modified=false;
}


void StepWidget_BaseGamma::Print()
{
	SaveChanges();
	float initg=gtk_spin_button_get_value(GTK_SPIN_BUTTON(initialgamma));

	// We render the Gamma Sweep pattern to be a little under 8 inches wide
	ImageSource *is=MakeGammaEvalSweep(colorants,1560,240,200);
	is=new ImageSource_ModifiedGamma(is,initg);
//	is=new ImageSource_Gamma(is,initg);

	// And apply the density correction too
	float initd=gui->gprinter.linearization.FindFloat("BaseDensity");
	ISDeviceNValue density(is->samplesperpixel,ISDataType(initd*IS_SAMPLEMAX));
	ImageSource_ScaleDensity dst(is,density);

	gui->Print(&dst,20,15,true);	// Bypass linearization since we're applying density manually.
}


// Callback function to mark the step as modified...
void StepWidget_BaseGamma::markmodified(GtkWidget *wid,gpointer data)
{
	StepWidget_BaseGamma *step=(StepWidget_BaseGamma *)data;
	step->modified=true;
}


// Callback function for the button
void StepWidget_BaseGamma::printclicked(GtkWidget *wid,gpointer data)
{
	StepWidget_BaseGamma *step=(StepWidget_BaseGamma *)data;
	try
	{
		step->Print();
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,step->gui->window);
	}
}

